from django.contrib import admin

from . import models


class ZendeskUserAdmin(admin.ModelAdmin):

    list_display = [
        "zendesk_id",
        "name",
        "email",
        "active",
        "role",
        "user",
        "created_at",
    ]
    list_filter = ["active", "role", "created_at"]
    raw_id_fields = ["user"]
    search_fields = ["zendesk_id", "name", "email"]

    def get_queryset(self, request):
        return models.ZendeskUser.objects.all().select_related("user")


class TicketAdmin(admin.ModelAdmin):

    list_display = [
        "zendesk_id",
        "requester",
        "subject",
        "status",
        "created_at",
        "updated_at",
    ]
    list_filter = ["status", "created_at", "updated_at"]
    # for Django <2.1
    raw_id_fields = ["requester"]
    # for Django >=2.1
    autocomplete_fields = ["requester"]
    search_fields = ["ticket__subject", "ticket__zendesk_id"]

    def get_queryset(self, request):
        return models.Ticket.objects.all().select_related("requester")


class CommentAdmin(admin.ModelAdmin):
    list_display = [
        "zendesk_id",
        "ticket_id",
        "author",
        "get_body",
        "public",
        "created_at",
    ]
    list_filter = ["public", "created_at"]
    # for Django <2.1
    raw_id_fields = ["ticket", "author"]
    # for Django >=2.1
    autocomplete_fields = ["ticket", "author"]

    def get_body(self, instance):
        return instance.plain_body or instance.body

    get_body.short_description = "Body"

    def get_queryset(self, request):
        return models.Comment.objects.all().select_related("ticket", "author")


class AttachmentAdmin(admin.ModelAdmin):

    list_display = ["zendesk_id", "comment_id", "file_name", "content_type", "inline"]
    list_filter = ["inline", "content_type"]
    # for Django <2.1
    raw_id_fields = ["comment"]

    def get_queryset(self, request):
        return models.Attachment.objects.all().prefetch_related("photos")


class PhotoAdmin(admin.ModelAdmin):

    list_display = ["zendesk_id", "attachment_id", "file_name", "content_type"]
    list_filter = ["content_type"]
    # for Django <2.1
    raw_id_fields = ["attachment"]

    def get_queryset(self, request):
        return models.Photo.objects.all()


class EventAdmin(admin.ModelAdmin):
    list_display = ["remote_ticket_id", "processing_ok", "created_at", "updated_at"]
    list_filter = ["created_at", "updated_at"]
    # for Django <2.1
    raw_id_fields = ["ticket"]
    # for Django >=2.1
    autocomplete_fields = ["ticket"]

    def processing_ok(self, instance):
        return False if instance.error else True

    processing_ok.boolean = True


admin.site.register(models.ZendeskUser, ZendeskUserAdmin)
admin.site.register(models.Ticket, TicketAdmin)
admin.site.register(models.Comment, CommentAdmin)
admin.site.register(models.Attachment, AttachmentAdmin)
admin.site.register(models.Photo, PhotoAdmin)
admin.site.register(models.Event, EventAdmin)



# Generated by Django 2.1.4 on 2019-03-27 20:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("zengo", "0001_initial")]

    operations = [
        migrations.CreateModel(
            name="Attachment",
            fields=[
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("zendesk_id", models.BigIntegerField(unique=True)),
                ("file_name", models.TextField()),
                ("content_url", models.URLField()),
                ("content_type", models.TextField()),
                ("size", models.BigIntegerField()),
                ("width", models.PositiveIntegerField(blank=True, null=True)),
                ("height", models.PositiveIntegerField(blank=True, null=True)),
                ("inline", models.BooleanField()),
            ],
        ),
        migrations.CreateModel(
            name="Photo",
            fields=[
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("zendesk_id", models.BigIntegerField(unique=True)),
                ("file_name", models.TextField()),
                ("content_url", models.URLField()),
                ("content_type", models.TextField()),
                ("size", models.BigIntegerField()),
                ("width", models.PositiveIntegerField(blank=True, null=True)),
                ("height", models.PositiveIntegerField(blank=True, null=True)),
                (
                    "attachment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="photos",
                        to="zengo.Attachment",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="comment",
            name="html_body",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="comment",
            name="plain_body",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="attachment",
            name="comment",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="attachments",
                to="zengo.Comment",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import

from django.dispatch import Signal


# fired any time a ticket is newly created
ticket_created = Signal(providing_args=["ticket", "context"])

# fired upon any change to a ticket beyond its initial creation
ticket_updated = Signal(providing_args=["ticket", "updates", "context"])



# -*- coding: utf-8 -*-
from __future__ import absolute_import

from django.core.exceptions import ValidationError
from django.http import HttpResponse
from django.http import HttpResponseBadRequest
from django.http import HttpResponseForbidden
from django.utils.crypto import constant_time_compare
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.base import View

from . import strings
from .service import get_processor
from .settings import app_settings


@method_decorator(csrf_exempt, name="dispatch")
class WebhookView(View):
    """Receive an update from Zendesk that a ticket has changed."""

    secret_name = "secret"
    secret = app_settings.WEBHOOK_SECRET

    def validate_secret(self):
        secret_given = self.request.GET.get(self.secret_name) or self.request.POST.get(
            self.secret_name
        )
        return secret_given is not None and constant_time_compare(
            secret_given, self.secret
        )

    def post(self, request):
        if not self.validate_secret():
            return HttpResponseForbidden(strings.secret_missing_or_wrong)
        processor = get_processor()
        try:
            event = processor.store_event(request.body.decode("utf-8"))
        except ValidationError as ve:
            return HttpResponseBadRequest(ve.message)
        processor.begin_processing_event(event)
        return HttpResponse()



# Generated by Django 2.1.4 on 2019-06-27 16:11

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("zengo", "0002_attachments")]

    operations = [
        migrations.AlterField(
            model_name="attachment",
            name="content_url",
            field=models.TextField(validators=[django.core.validators.URLValidator()]),
        ),
        migrations.AlterField(
            model_name="photo",
            name="content_url",
            field=models.TextField(validators=[django.core.validators.URLValidator()]),
        ),
        migrations.AlterField(
            model_name="ticket",
            name="url",
            field=models.TextField(
                blank=True,
                null=True,
                validators=[django.core.validators.URLValidator()],
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import

from django.conf import settings


DEFAULTS = {"SERVICE_CLASS": None, "WEBHOOK_SECRET": None, "PROCESSOR_CLASS": None}


class AppSettings(object):
    def __init__(self, prefix, defaults):
        self.prefix = prefix
        self.defaults = defaults

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError('Invalid app setting: "{}"'.format(attr))
        return getattr(settings, "{}_{}".format(self.prefix, attr), self.defaults[attr])


app_settings = AppSettings("ZENGO", DEFAULTS)
